using System.Collections.Concurrent;
using Todo.Core.Application.Services.UserConnectionStore;

namespace Todo.Infrastructure.UserConnectionStore;

internal class InMemoryUserConnectionStore : IUserConnectionStore
{
    private static readonly ConcurrentDictionary<string, List<string>>
        ConnectedUsers = new();

    public Task AddAsync(string userId, string connectionId)
    {
        // Try to get a List of existing user connections from the cache
        ConnectedUsers.TryGetValue(userId, out var existingUserConnectionIds);

        // happens on the very first connection from the user
        existingUserConnectionIds ??= new List<string>();

        // First add to a List of existing user connections (i.e. multiple web browser tabs)
        existingUserConnectionIds.Add(connectionId);

        // Add to the global dictionary of connected users
        ConnectedUsers.TryAdd(userId, existingUserConnectionIds);

        return Task.CompletedTask;
    }

    public Task<IEnumerable<string>> GetConnectionsAsync(string userId)
    {
        ConnectedUsers.TryGetValue(userId, out var connections);
        return Task.FromResult(
            connections is null
                ? new List<string>().AsEnumerable()
                : connections.AsEnumerable());
    }

    public Task RemoveAsync(string userId, string connectionId)
    {
        ConnectedUsers.TryGetValue(userId, out var existingUserConnectionIds);

        // remove the connection id from the List 
        try
        {
            existingUserConnectionIds?.Remove(connectionId);
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }

        // If there are no connection ids in the List, delete the user from the global cache (ConnectedUsers).
        if (existingUserConnectionIds?.Count == 0)
            // if there are no connections for the user,
            // just delete the userName key from the ConnectedUsers concurent dictionary
            ConnectedUsers.TryRemove(userId, out _);

        return Task.CompletedTask;
    }
}